unit uutlEnumeratorTests;

{$mode objfpc}{$H+}
{$modeswitch nestedprocvars}

interface

uses
  Classes, SysUtils, TestFramework,
  uutlEnumerator;

type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  TutlEnumeratorTests = class(TTestCase)
  published
    procedure ArrayEnumerator;
    procedure ArrayEnumerator_Reverse;
    procedure ArrayEnumerator_Count;
    procedure ArrayEnumerator_Skip;
    procedure ArrayEnumerator_Take;
    procedure ArrayEnumerator_Skip_Reverse;
    procedure ArrayEnumerator_Take_Reverse;
    procedure ArrayEnumerator_Reverse_Skip;
    procedure ArrayEnumerator_Reverse_Take;
    procedure ArrayEnumerator_Where;
    procedure ArrayEnumerator_Select;
  end;

implementation

uses
  uutlFilter;

type
  TIntCalbackFilter = specialize TutlCalbackFilter<Integer>;
  TIntArrEnumerator = specialize TutlArrayEnumerator<Integer>;
  TFloatArrEnumerator = specialize TutlArrayEnumerator<Single>;

function CreateArrayEnumerator(const aSize: Integer): TIntArrEnumerator.IEnumerator;
var
  arr: array of Integer;
  i: Integer;
begin
  SetLength(arr, aSize);
  for i := low(arr) to high(arr) do
    arr[i] := i + 1;
  result := TIntArrEnumerator.Create(arr);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlEnumeratorTests///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(5);
  AssertTrue  (e.MoveNext);
  AssertEquals(1, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(2, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(3, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(5, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Reverse;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(5)
    .Reverse;
  AssertTrue  (e.MoveNext);
  AssertEquals(5, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(3, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(2, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(1, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Count;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(5);
  AssertEquals(5, e.Count);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Skip;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Skip(5);
  AssertTrue  (e.MoveNext);
  AssertEquals(6, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(7, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(8, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(9, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(10, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Take;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Take(5);
  AssertTrue  (e.MoveNext);
  AssertEquals(1, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(2, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(3, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(5, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Skip_Reverse;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Skip(5)
    .Reverse;
  AssertTrue  (e.MoveNext);
  AssertEquals(10, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(9, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(8, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(7, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(6, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Take_Reverse;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Take(5)
    .Reverse;
  AssertTrue  (e.MoveNext);
  AssertEquals(5, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(3, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(2, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(1, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Reverse_Skip;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Reverse
    .Skip(5);
  AssertTrue  (e.MoveNext);
  AssertEquals(5, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(3, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(2, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(1, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Reverse_Take;
var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Reverse
    .Take(5);
  AssertTrue  (e.MoveNext);
  AssertEquals(10, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(9, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(8, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(7, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(6, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Where;

  function IsEven(constref i: Integer): Boolean;
  begin
    result := (i mod 2) = 0;
  end;

var
  e: TIntArrEnumerator.IEnumerator;
begin
  e := CreateArrayEnumerator(10)
    .Where(TIntCalbackFilter.Create(@IsEven));
  AssertTrue  (e.MoveNext);
  AssertEquals(2, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(6, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(8, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(10, e.Current);
  AssertFalse (e.MoveNext);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
procedure TutlEnumeratorTests.ArrayEnumerator_Select;

  function ConvertToFloat(constref a: Integer): Single;
  begin
    result := Single(a);
  end;

var
  e: TFloatArrEnumerator.IEnumerator;
begin
  e := specialize TutlSelectEnumerator<Integer, Single>.Create(
    CreateArrayEnumerator(5),
    specialize TutlCalbackSelector<Integer, Single>.Create(@ConvertToFloat));
  AssertTrue  (e.MoveNext);
  AssertEquals(1.0, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(2.0, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(3.0, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(4.0, e.Current);
  AssertTrue  (e.MoveNext);
  AssertEquals(5.0, e.Current);
  AssertFalse (e.MoveNext);
end;

initialization
  RegisterTest(TutlEnumeratorTests.Suite);

end.

