unit uutlComparer;

{$mode objfpc}{$H+}
{$modeswitch nestedprocvars}

interface

uses
  Classes, SysUtils,
  uutlInterfaces;

type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  generic TutlEqualityComparer<T> = class(
    TInterfacedObject,
    specialize IutlEqualityComparer<T>)

  public
    function EqualityCompare(constref i1, i2: T): Boolean;
  end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  generic TutlEqualityCompareEvent<T>  = function(constref i1, i2: T): Boolean;
  generic TutlEqualityCompareEventO<T> = function(constref i1, i2: T): Boolean of object;
  generic TutlEqualityCompareEventN<T> = function(constref i1, i2: T): Boolean is nested;

  generic TutlCalbackEqualityComparer<T> = class(
    TInterfacedObject,
    specialize IutlEqualityComparer<T>)

  private type
    TEqualityCompareEventType = (eetNormal, eetObject, eetNested);

  public type
    TCompareEvent = specialize TutlEqualityCompareEvent<T>;
    TCompareEventO = specialize TutlEqualityCompareEventO<T>;
    TCompareEventN = specialize TutlEqualityCompareEventN<T>;

  strict private
    fType: TEqualityCompareEventType;
    fEvent: TCompareEvent;
    fEventO: TCompareEventO;
    fEventN: TCompareEventN;

  public
    function EqualityCompare(constref i1, i2: T): Boolean;

    { HINT: you need to activate "$modeswitch nestedprocvars" when you want to use nested callbacks }
    constructor Create(const aEvent: TCompareEvent); overload;
    constructor Create(const aEvent: TCompareEventO); overload;
    constructor Create(const aEvent: TCompareEventN); overload;
  end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  generic TutlComparer<T> = class(
    specialize TutlEqualityComparer<T>,
    specialize IutlEqualityComparer<T>,
    specialize IutlComparer<T>)

  public
    function Compare(constref i1, i2: T): Integer;
  end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  generic TutlCompareEvent<T>  = function(constref i1, i2: T): Integer;
  generic TutlCompareEventO<T> = function(constref i1, i2: T): Integer of object;
  generic TutlCompareEventN<T> = function(constref i1, i2: T): Integer is nested;

  generic TutlCallbackComparer<T> = class(
    TInterfacedObject,
    specialize IutlEqualityComparer<T>,
    specialize IutlComparer<T>)

  private type
    TCompareEventType = (cetNormal, cetObject, cetNested);

  public type
    TCompareEvent  = specialize TutlCompareEvent<T>;
    TCompareEventO = specialize TutlCompareEventO<T>;
    TCompareEventN = specialize TutlCompareEventN<T>;

  strict private
    fType: TCompareEventType;
    fEvent: TCompareEvent;
    fEventO: TCompareEventO;
    fEventN: TCompareEventN;

  public
    function Compare(constref i1, i2: T): Integer;
    function EqualityCompare(constref i1, i2: T): Boolean;

    { HINT: you need to activate "$modeswitch nestedprocvars" when you want to use nested callbacks }
    constructor Create(const aEvent: TCompareEvent); overload;
    constructor Create(const aEvent: TCompareEventO); overload;
    constructor Create(const aEvent: TCompareEventN); overload;
  end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
operator <(const i1, i2: TObject): Boolean; inline;
operator >(const i1, i2: TObject): Boolean; inline;

implementation

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Helper////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
operator <(const i1, i2: TObject): Boolean; inline;
begin
  result := Pointer(i1) < Pointer(i2);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
operator >(const i1, i2: TObject): Boolean; inline;
begin
  result := Pointer(i1) > Pointer(i2);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlEqualityComparer//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlEqualityComparer.EqualityCompare(constref i1, i2: T): Boolean;
begin
  result := (i1 = i2);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlCalbackEqualityComparer///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlCalbackEqualityComparer.EqualityCompare(constref i1, i2: T): Boolean;
begin
  result := false;
  case fType of
    eetNormal: result := fEvent (i1, i2);
    eetObject: result := fEventO(i1, i2);
    eetNested: result := fEventN(i1, i2);
  else
    raise Exception.Create('invalid or unknown callback type');
  end;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackEqualityComparer.Create(const aEvent: TCompareEvent);
begin
  inherited Create;
  fType  := eetNormal;
  fEvent := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackEqualityComparer.Create(const aEvent: TCompareEventO);
begin
  inherited Create;
  fType   := eetObject;
  fEventO := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackEqualityComparer.Create(const aEvent: TCompareEventN);
begin
  inherited Create;
  fType   := eetNested;
  fEventN := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlComparer//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlComparer.Compare(constref i1, i2: T): Integer;
begin
  if (i1 < i2) then
    result := -1
  else if (i1 > i2) then
    result := 1
  else
    result := 0;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlCallbackComparer//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlCallbackComparer.Compare(constref i1, i2: T): Integer;
begin
  result := 0;
  case fType of
    cetNormal: result := fEvent (i1, i2);
    cetObject: result := fEventO(i1, i2);
    cetNested: result := fEventN(i1, i2);
  else
    raise Exception.Create('invalid or unknown callback type');
  end;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlCallbackComparer.EqualityCompare(constref i1, i2: T): Boolean;
begin
  result := (Compare(i1, i2) = 0);
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCallbackComparer.Create(const aEvent: TCompareEvent);
begin
  inherited Create;
  fType   := cetNormal;
  fEvent  := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCallbackComparer.Create(const aEvent: TCompareEventO);
begin
  inherited Create;
  fType   := cetObject;
  fEventO := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCallbackComparer.Create(const aEvent: TCompareEventN);
begin
  inherited Create;
  fType   := cetNested;
  fEventN := aEvent;
end;

end.

