unit uutlFilter;

{$mode objfpc}{$H+}
{$modeswitch nestedprocvars}

interface

uses
  Classes, SysUtils,
  uutlInterfaces;

type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  generic TutlFilterEvent<T>  = function(constref i: T): Boolean;
  generic TutlFilterEventO<T> = function(constref i: T): Boolean of object;
  generic TutlFilterEventN<T> = function(constref i: T): Boolean is nested;

  generic TutlCalbackFilter<T> = class(
    TInterfacedObject,
    specialize IutlFilter<T>)

  private type
    TFilterEventType = (fetNormal, fetObject, fetNested);

  public type
    TFilterEvent  = specialize TutlFilterEvent<T>;
    TFilterEventO = specialize TutlFilterEventO<T>;
    TFilterEventN = specialize TutlFilterEventN<T>;

  strict private
    fType: TFilterEventType;
    fEvent: TFilterEvent;
    fEventO: TFilterEventO;
    fEventN: TFilterEventN;

  public
    function Filter(constref i: T): Boolean;

    { HINT: you need to activate "$modeswitch nestedprocvars" when you want to use nested callbacks }
    constructor Create(const aEvent: TFilterEvent); overload;
    constructor Create(const aEvent: TFilterEventO); overload;
    constructor Create(const aEvent: TFilterEventN); overload;
  end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  generic TutlSelectEvent <Tin, Tout> = function(constref i: Tin): Tout;
  generic TutlSelectEventO<Tin, Tout> = function(constref i: Tin): Tout of object;
  generic TutlSelectEventN<Tin, Tout> = function(constref i: Tin): Tout is nested;

  generic TutlCalbackSelector<Tin, Tout> = class(
    TInterfacedObject,
    specialize IutlSelector<Tin, Tout>)

  private type
    TSelectEventType = (setNormal, setObject, setNested);

  public type
    TSelectEvent  = specialize TutlSelectEvent <Tin, Tout>;
    TSelectEventO = specialize TutlSelectEventO<Tin, Tout>;
    TSelectEventN = specialize TutlSelectEventN<Tin, Tout>;

  strict private
    fType: TSelectEventType;
    fEvent: TSelectEvent;
    fEventO: TSelectEventO;
    fEventN: TSelectEventN;

  public
    function Select(constref i: Tin): Tout;

    { HINT: you need to activate "$modeswitch nestedprocvars" when you want to use nested callbacks }
    constructor Create(const aEvent: TSelectEvent); overload;
    constructor Create(const aEvent: TSelectEventO); overload;
    constructor Create(const aEvent: TSelectEventN); overload;
  end;

implementation

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlCalbackFilter/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlCalbackFilter.Filter(constref i: T): Boolean;
begin
  result := false;
  case fType of
    fetNormal: result := fEvent (i);
    fetObject: result := fEventO(i);
    fetNested: result := fEventN(i);
  else
    raise Exception.Create('invalid or unknown callback type');
  end;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackFilter.Create(const aEvent: TFilterEvent);
begin
  inherited Create;
  fType  := fetNormal;
  fEvent := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackFilter.Create(const aEvent: TFilterEventO);
begin
  inherited Create;
  fType   := fetObject;
  fEventO := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackFilter.Create(const aEvent: TFilterEventN);
begin
  inherited Create;
  fType   := fetNested;
  fEventN := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TutlCalbackSelector///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function TutlCalbackSelector.Select(constref i: Tin): Tout;
begin
  case fType of
    setNormal: result := fEvent (i);
    setObject: result := fEventO(i);
    setNested: result := fEventN(i);
  else
    raise Exception.Create('invalid or unknown callback type');
  end;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackSelector.Create(const aEvent: TSelectEvent);
begin
  inherited Create;
  fType  := setNormal;
  fEvent := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackSelector.Create(const aEvent: TSelectEventO);
begin
  inherited Create;
  fType   := setObject;
  fEventO := aEvent;
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
constructor TutlCalbackSelector.Create(const aEvent: TSelectEventN);
begin
  inherited Create;
  fType   := setNested;
  fEventN := aEvent;
end;

end.

